/**************************/
/* constructionLongueur.c */
/**************************/

/*
   Trigger permettant d'inserer la valeur de la longueur
   de la route issue de la relation routes dans une
   relation routesTrigger sur la colonne longueur
   Modification : lancement d'un ordre UPDATE
*/

#include "postgres.h"
#include "executor/spi.h"       /* nécessaire pour fonctionner avec SPI */
#include "commands/trigger.h"   /* ... et les déclencheurs */

#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

extern Datum constructionlongueur(PG_FUNCTION_ARGS);


Datum constructionlongueur(PG_FUNCTION_ARGS)
/********************************************/

{
  TriggerData *trigdata = (TriggerData *) fcinfo->context;
  HeapTuple   rettuple;

  bool        checkNULL = false;
  char        *when;

  TupleDesc   tupdesc;
  int         ret;
  bool        isNULL;

  char        ordre[100];
  char        relation[100];
  int         identifiant;
  double      longueur;
  int         numeroAttribut;
  char        *valeurAttribut;

  /* on s'assure que la fonction est appelée en tant que déclencheur */
  if (!CALLED_AS_TRIGGER(fcinfo))
    elog(ERROR, "constructionlongueur: not called by trigger manager");

  /* nuplet à retourner à l'exécuteur */
  if (TRIGGER_FIRED_BY_UPDATE(trigdata->tg_event))
    rettuple = trigdata->tg_newtuple;
  else
    rettuple = trigdata->tg_trigtuple;

  /* vérification des valeurs NULL */
  if (!TRIGGER_FIRED_BY_DELETE(trigdata->tg_event)
      && TRIGGER_FIRED_BEFORE(trigdata->tg_event))
    checkNULL = true;

  if (TRIGGER_FIRED_BEFORE(trigdata->tg_event))
    when = "before";
  else
    when = "after ";

  tupdesc = trigdata->tg_relation->rd_att;

  /* connexion au gestionnaire SPI */
  if ((ret = SPI_connect()) < 0)
    elog(ERROR, "constructionlongueur (fired %s): SPI_connect returned %d", when, ret);



/********************************************/
/* Specifique schema BD : attribut cle : id */
/********************************************/

  numeroAttribut = SPI_fnumber (trigdata->tg_relation->rd_att, "id");
  if (numeroAttribut == SPI_ERROR_NOATTRIBUTE) {
      elog (ERROR, "Numero attribut non valide ");
  } else {
      elog (INFO, "Numero attribut :%d: ", numeroAttribut);
  }

  valeurAttribut = SPI_getvalue(trigdata->tg_trigtuple, trigdata->tg_relation->rd_att, numeroAttribut);
  elog (INFO, "Valeur attribut :%s: ", valeurAttribut);

  identifiant = atoi(valeurAttribut);
  sprintf (ordre, "SELECT ST_length(the_geom) FROM routes Where id = %d", identifiant);
  SPI_pfree (valeurAttribut);

  /* obtient la longueur de la route */
  ret = SPI_exec(ordre, 0);
  if (ret < 0)
    elog(ERROR, "constructionlongueur (fired %s): SPI_exec returned %d", when, ret);

  /* ST_length renvoie un type double */
  longueur = DatumGetFloat8(SPI_getbinval(SPI_tuptable->vals[0],
				  SPI_tuptable->tupdesc,
				  1,
				  &isNULL));

  elog (INFO, "constructionlongueur : la longueur est %f ", longueur);


  /* On obtient la relation */
  /**************************/
  strcpy(relation , SPI_getrelname(trigdata->tg_relation));

  /* On modifie le tuple  car on est en mode after */
  /*************************************************/
  sprintf (ordre,"UPDATE %s set longueur = %f where id = %d", relation, longueur, identifiant);
  elog(INFO, "ordre envoye :%s: ", ordre);

  ret = SPI_exec(ordre, 0);
  if (ret < 0)
    elog(ERROR, "constructionlongueur (fired %s): SPI_exec returned %d", when, ret);

  SPI_finish();

  if (checkNULL)
    {
      SPI_getbinval(rettuple, tupdesc, 1, &isNULL);
      if (isNULL)
	  rettuple = NULL;
    }

  return PointerGetDatum(rettuple);

} /* constructionlongueur */
